package background;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import application.Profile;

public class LoadSample {
	public static String CHECK_LIST = "./checklist";
	public static String SOCIAL_LIST = "./socialRelations";
	public static String TWEET_DIR = "./tweets/";
	public static String PROFILE_DIR = "./profiles/";
	
	public BufferedReader idis; 
	public BufferedReader sois;
	public BufferedReader pris;
	public BufferedReader twis;
	
	public int numOfSamples = 0;
	
	public ArrayList<String> keys = new ArrayList<String>();
	public HashMap<String, ArrayList<String>> id2follow = new HashMap<String, ArrayList<String>>();
	public HashMap<String, Profile> id2profile = new HashMap<String, Profile>();
	public HashMap<String, ArrayList<String>> id2tweet = new HashMap<String, ArrayList<String>>();
	
	public void loadTestInfo() {
		String stId;
		try {
			idis = new BufferedReader(new FileReader(CHECK_LIST));
			sois = new BufferedReader(new FileReader(SOCIAL_LIST));
			
			while((stId = idis.readLine()) != null) {
				stId = stId.trim();
				keys.add(stId);
				loadProfile(stId);
				loadTweet(stId);
				numOfSamples += 1;
			}
			
			loadSocialRalation();
			
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
	}
	
	public void loadProfile(String stId) {
		try {
			pris = new BufferedReader(new FileReader(PROFILE_DIR+stId+"_profile"));
			String uid  = pris.readLine().trim();
			String nam = pris.readLine().trim();
			String bio  = pris.readLine().trim();
			String loc  = pris.readLine().trim();
			String web =  pris.readLine().trim();
			
			id2profile.put(uid, new Profile(uid, nam, loc, web, bio));
			
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
	}
	
	public void loadTweet(String stId) {
		try {
			twis = new BufferedReader(new FileReader(TWEET_DIR+stId+"_tweet"));
			String tweet;
			ArrayList<String> al = new ArrayList<String>();
			while((tweet = twis.readLine()) != null) {
				tweet = tweet.trim();
				al.add(tweet);
			}
			
			id2tweet.put(stId, al);
			
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
	}
	
	public void loadSocialRalation() {
		try {
			sois = new BufferedReader(new FileReader(SOCIAL_LIST));
			String socio;
			String[] ot;
			while((socio = sois.readLine()) != null) {
				ot = socio.trim().split(" ");
				ArrayList<String> a = id2follow.get(ot[0]);
				if(a == null) {
					a = new ArrayList<String>();
					a.add(ot[1]);
					id2follow.put(ot[0], a);
				} else {
					a.add(ot[1]);
				}
			}
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
	}
	
	public void printId2follow() {
		Set<String> keyset = id2follow.keySet();
		Iterator<String> iterator  = keyset.iterator();
		while(iterator.hasNext()) {
			String key = iterator.next();
			System.out.println("-----------" + key + "-----------");
			ArrayList<String> al = id2follow.get(key);
			printArrayList(al);
		}
	}
	
	public void printId2profile() {
		Set<String> keyset = id2profile.keySet();
		Iterator<String> iterator  = keyset.iterator();
		while(iterator.hasNext()) {
			String key = iterator.next();
			System.out.println("-----------" + key + "-----------");
			Profile p = id2profile.get(key);
			p.print();
		}
	}
	
	public void printId2tweet() {
		Set<String> keyset = id2tweet.keySet();
		Iterator<String> iterator  = keyset.iterator();
		while(iterator.hasNext()) {
			String key = iterator.next();
			System.out.println("-----------" + key + "-----------");
			ArrayList<String> al = id2tweet.get(key);
			printArrayList(al);
		}
	}
	
	public void printArrayList(ArrayList<String> alist) {
		Iterator<String> iterator = alist.iterator();
		while(iterator.hasNext()) {
			System.out.println(iterator.next());
		}
	}
	
	public int getNumOfSample() {
		return numOfSamples;
	}
	
	public ArrayList<String> getFollowees(String uid) {
		return id2follow.get(uid);
	}
	
	public Profile getProfile(String uid) {
		return id2profile.get(uid);
	}
	
	public ArrayList<String> getTweets(String uid) {
		return id2tweet.get(uid);
	}
	
	public ArrayList<String> getKeys() {
		return keys;
	}
	
	public static void main(String[] args) {
		LoadSample ls = new LoadSample();
		ls.loadTestInfo();
		ls.printId2follow();
		ls.printId2profile();
		ls.printId2tweet();
	}
}
