package database;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import toolbox.Account;

public class AccountXMLDB {
	private String path;
	private String rootName;
	
	public AccountXMLDB(String dir, String path, String root) {
		File file = new File(path);
		this.path = path;
		this.rootName = root;
		
		try {
			if(file.exists() == false) {
				File fDir = new File(dir);
				if(fDir.exists() == false) {
					fDir.mkdir();
				}
				file.createNewFile();
				createXmlDoc();
			}			
			
		} catch(FileNotFoundException e1) {
			System.out.println("FileNotFoundException @ createXmlDoc in AccountXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ createXmlDoc in AccountXMLDB");
		}
	}
	
	private void createXmlDoc() throws FileNotFoundException, IOException {
		Element root = new Element(rootName);
		Document Doc = new Document(root);
		
		XMLOutputter XMLOut = new XMLOutputter();
		XMLOut.output(Doc, new FileOutputStream(path));
	}
	
	/** 
	 *  return: false - account already exist!
	 *          true  - success
	 */
	public synchronized boolean addAccount(Account account) {
		try {
			Account ac = getAccount(account.getUsername());
			if(ac != null) {
				System.out.println("Account already exist!");
				return false;
			}
			
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
			
			Element element0 = new Element("username");
			element0.setAttribute("username", account.getUsername());
			//Element element1 = new Element("password");
			//element1.setText(account.getPassword());
			Element element2 = new Element("userId");
			element2.setText(account.getUserId());
			//element0.addContent(element1);
			element0.addContent(element2);

			root.addContent(element0);
			doc.setRootElement(root);

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ addProfile in AccountXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ addProfile in AccountXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ addProfile in AccountXMLDB");
		}
		return true;
	}
	
	public synchronized String getUserId(String username) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if ( e.getAttributeValue("username").compareTo(username) == 0 ) {
					return e.getChildText("userId");
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getProfile in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ getProfile in ProfileXMLDB");
		}
		return null;
	}
	
	public synchronized Account getAccount(String username) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if ( e.getAttributeValue("username").compareTo(username) == 0 ) {
					return new Account(e.getAttributeValue("username"), e.getChildText("userId"));
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getProfile in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ getProfile in ProfileXMLDB");
		}
		return null;
	}
	
	public synchronized void xmlParsePrint() {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				System.out.println("acc--"+e.getAttributeValue("username")+" "+e.getChildText("password")+" "+e.getChildText("userId"));
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException in xmlParsePrint!");
		} catch(IOException e2) {
			System.out.println("IOException in xmlParsePrint!");
		}
	}
	
	public synchronized void deleteProfile(String username) throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		InputStream file = new FileInputStream(path);
		Document doc = builder.build(file);

		Element root = doc.getRootElement();

		List<Element> list = root.getChildren();

		for (Element e : list) {
			if (e.getAttributeValue("username").compareTo(username) == 0) {
				root.removeContent(e);
				break;// ?? What should we do?
			}
		}

		XMLOutputter out = new XMLOutputter();
		out.output(doc, new FileOutputStream(path));
	}
	
	/**
	 * @param account
	 * @return 
	 * 1: account not found
	 * 2: password mismatching!
	 * 3: pass checking
	 */
	
	public int checkAccount(Account account) {
		return 3;
		/*
		Account a = getAccount(account.getUsername());
		if(a == null) {
			return 1;
		}
		if(a.getPassword().compareTo(account.getPassword()) == 0) {
			return 3;
		} else {
			return 2;
		}
		*/
	}
}
