package database;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import application.Status;


public class StatusXMLDB {
	private String path;
	private String rootName;

	public StatusXMLDB(String dir, String path, String root) {
		File file = new File(path);
		this.path = path;
		this.rootName = root;
		
		try {
			if(file.exists() == false) {
				File fDir = new File(dir);
				if(fDir.exists() == false) {
					fDir.mkdir();
				}
				file.createNewFile();
				creatXmlDoc();
			}
		} catch(FileNotFoundException e1) {
			System.out.println("FileNotFoundException @ createXmlDoc in StatusXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ createXmlDoc in StatusXMLDB");
		}
	}
	
	private void creatXmlDoc() {
		Element root = new Element(rootName);
		Document Doc = new Document(root);

		// Element elements = new Element("status");
		// elements.setAttribute("ID", new Integer(0).toString());
		// elements.addContent(new Element("text").setText("Welcome!"));
		// elements.addContent(new Element("username").setText("name" + i));
		// elements.addContent(new Element("password").setText("@" + i+
		// ".com"));
		// root.addContent(elements);
		try {
			XMLOutputter XMLOut = new XMLOutputter();
			XMLOut.output(Doc, new FileOutputStream(path));
		} catch(FileNotFoundException e1) {
			System.out.print("FileNotFoundException@creatXmlDoc in StatusXMLDB");
		} catch(IOException e2) {
			System.out.print("IOException@creatXmlDoc in StatusXMLDB");
		}
	}

	public synchronized void xmlParsePrint() throws JDOMException, IOException {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				//System.out.println("Status ID = " + e.getAttributeValue("statusId"));
				//System.out.println("Published by: " + e.getChildText("username"));
				//System.out.println("Published at: " + e.getChildText("time"));
				//System.out.println("Status: " + e.getChildText("text"));
				System.out.println(e.getAttributeValue("statusId")+" "+e.getChildText("username")+" "+e.getChildText("text")+" "+e.getChildText("time"));
				//System.out.println(root.indexOf(e));
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ xmlParsePrint in StatusXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ xmlParsePrint in StatusXMLDB");
		}
	}

	public synchronized void addStatus(Status status) {
		try {
			if(exist(status) == true) {
				return;
			}
			
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();

			Element element = new Element("statusId");
			element.setAttribute("statusId", status.getStatusId());

			Element element1 = new Element("username");
			element1.setText(status.getAuthor());
		
			Element element2 = new Element("userId");
			element2.setText(status.getAuthorId());

			Element element3 = new Element("time");
			element3.setText(Long.toString(status.getTime()));

			Element element4 = new Element("text");
			element4.setText(status.getText());

			element.addContent(element1);
			element.addContent(element2);
			element.addContent(element3);
			element.addContent(element4);

			root.addContent(findIndex(root, Long.valueOf(status.getTime())), element);
			doc.setRootElement(root);

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		} catch(JDOMException e1) {
			System.out.println("JDOMException in addStatus!");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException in addStatus!");
		} catch(IOException e3) {
			System.out.println("IOException in addStatus!");
		}
	}
	
	private synchronized int findIndex(Element r, long t) {
		List<Element> ll = r.getChildren();
		for (Element e : ll) {
			if(t >= Long.valueOf(e.getChildText("time"))) {
				return r.indexOf(e);
			}
		}
		return ll.size();
	}
	
	public synchronized void deleteStatus(long statusId) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);

			Element root = doc.getRootElement();

			List<Element> list = root.getChildren();

			for (Element e : list) {
				if (Integer.parseInt(e.getAttributeValue("statusId")) == statusId) {
					root.removeContent(e);
					break;// ?? What should we do?
				}
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		} catch (JDOMException e1) {
			System.out.println("JDOMException in deleteStatus!");
		} catch (IOException e2) {
			System.out.println("IOException in deleteStatus!");
		}
	}

	public synchronized Status[] getRecentStatus(int bulk) {
		LinkedList<Status> sll = new LinkedList<Status>();
		Status[] status;
		while(true) {
			try {
				SAXBuilder builder = new SAXBuilder();
				InputStream file = new FileInputStream(path);
				Document document = builder.build(file);
				Element root = document.getRootElement();
				List<Element> list = root.getChildren();
				int i = 0;
				for (Element e : list) {
					sll.addLast( new Status(
							e.getAttributeValue("statusId"),
							e.getChildText("text"),
							e.getChildText("username"),
							e.getChildText("userId"),
							Long.valueOf(e.getChildText("time")).longValue()
							));
					i += 1;
					if(i >= bulk) {
						break;
					}
				}
				break;
			} catch(JDOMException e1) {
				//System.out.println("JDOMException @ getRecentStatus in StatusXMLDB");
				continue;
			} catch (IOException e2) {
				System.out.println("IOException @ getRecentStatus in StatusXMLDB");
				continue;
			}
		}
		status = new Status[sll.size()];
		Iterator<Status> iterator = sll.iterator();
		int i = 0;
		while(iterator.hasNext()) {
			status[i] = iterator.next();
			i = i+1;
		}
		return status;
	}
	
	public synchronized Status[] getStatuses(long time) {
		LinkedList<Status> sll = new LinkedList<Status>();
		Status[] status;
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				long t = Long.valueOf(e.getChildText("time"));
				if(t > time) {
					sll.addLast(new Status(
							e.getAttributeValue("statusId"),
					        e.getChildText("text"),
							e.getChildText("username"),
							e.getChildText("userId"),
							Long.valueOf(e.getChildText("time")).longValue()
							));
				} else {
					break;
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getRecentStatus in StatusXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ getRecentStatus in StatusXMLDB");
		}
		//Copy the list to the array
		if(sll.size() == 0) {
			return null;
		}
		status = new Status[sll.size()];
		Iterator<Status> iterator = sll.iterator();
		int i = 0;
		while(iterator.hasNext()) {
			status[i] = iterator.next();
			i = i+1;
		}
		return status;
	}
	
	public synchronized static void printStatuses(Status[] statuses) {
		for(int i=0; i<statuses.length; i++) {
			System.out.println(statuses[i].getStatusId()+" "+statuses[i].getAuthor()+" "+statuses[i].getText()+" "+statuses[i].getTime());
		}
	}
	
	public static void printStatuses(LinkedList<Status> statuses) {
		Iterator<Status> iterator = statuses.iterator();
		Status ts;
		while(iterator.hasNext()) {
			ts = iterator.next();
			System.out.println(ts.getStatusId()+" "+ts.getAuthor()+" "+ts.getText()+" "+ts.getTime());
		}		
	}
	
	//Whether a status exists in the XML-based DB
	public synchronized boolean exist(Status s) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				//System.out.println("///" + s.getTime());
				//System.out.println("///" + e.getChildText("time"));
				//System.out.println("///" + s.getStatusId());
				//System.out.println("///" + e.getAttributeValue("statusId"));
				
				if(Long.valueOf(s.getTime()).longValue() > Long.valueOf(e.getChildText("time")).longValue()) {
					return false;
				} else if(Long.valueOf(s.getTime()).longValue() == Long.valueOf(e.getChildText("time")).longValue()) {
					if(s.getStatusId().compareTo(e.getAttributeValue("statusId"))==0) {
						return true;
					}
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ xmlParsePrint in StatusXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ xmlParsePrint in StatusXMLDB");
		}
		
		return false;
	}
	
	public synchronized long getLastTime() {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			Element e = root.getChild("statusId");
			if(e != null) {
				return Long.valueOf(e.getChildText("time"));
			} else {
				return 0;
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getLastTime in StatusXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ getLastTime in StatusXMLDB");
		}
		//Never get here!
		return 0;
	}
	
	// Currently, we do not need update!!
	/**
	 * public static void updateStatus(int id) throws JDOMException, IOException
	 * { SAXBuilder builder = new SAXBuilder(); InputStream file = new
	 * FileInputStream("xml/po.xml"); Document doc = builder.build(file);
	 * 
	 * Element root = doc.getRootElement();
	 * 
	 * List<Element> list = root.getChildren(); for(Element e:list) {
	 * if(Integer.parseInt(e.getAttributeValue("ID")) == id) {
	 * System.out.println("---------We find it!-----------");
	 * e.getChild("username").setText("111111111");
	 * e.getChild("password").setText("password"); } }
	 * 
	 * XMLOutputter out = new XMLOutputter(); out.output(doc, new
	 * FileOutputStream("xml/po.xml")); }
	 */

	public static void main(String ars[]) throws JDOMException, IOException {
		/*String dir = "xml/" + "hello";
		
		StatusXMLDB xmlDB = new StatusXMLDB(dir, dir+"/status.xml", "myStatus");
		xmlDB.creatXmlDoc();
		xmlDB.addStatus(new Status("123", "fuck you", "Tianyin", "1243", 121212));
		xmlDB.addStatus(new Status("234", "fuck you", "Tianyin", "1243", 121223));
		xmlDB.addStatus(new Status("345", "fuck you", "Tianyin", "1243", 121234));
		xmlDB.addStatus(new Status("456", "fuck you", "Tianyin", "1243", 121245));
		xmlDB.xmlParsePrint();
		
		System.out.println("-------------------------------------");
		
		xmlDB.addStatus(new Status("567", "fuck you", "Tianyin", "1243", 121233));
		
		xmlDB.addStatus(new Status("234", "fuck you", "Tianyin", "1243", 121223));
		xmlDB.addStatus(new Status("345", "fuck you", "Tianyin", "1243", 121234));
		xmlDB.addStatus(new Status("456", "fuck you", "Tianyin", "1243", 121245));
		
		//printStatuses(xmlDB.getRecentStatus(3));
		xmlDB.xmlParsePrint();
		
		//xmlDB.addStatus(new Status("234", "fuck you", "Tianyin", "1243", "121223"));
		
		//System.out.println();
		//printStatuses(xmlDB.getStatuses(121000));
		
		//xmlDB.xmlParsePrint();
		 */
		
		String dir = "xml/0x113601";
		StatusXMLDB xmlDB = new StatusXMLDB(dir, dir+"/mystatus.xml", "myStatus");
		//xmlDB.xmlParsePrint();
		System.out.println(xmlDB.getLastTime());
	}

}
