package guinterface;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;

import org.jfree.ui.RefineryUtilities;

import application.Cuckoo;


public class FunctionPanel extends JPanel {
	private static final long serialVersionUID = 10250320101712L;
	
	private Cuckoo cuckoo;
	
	private JButton tweetButton    = new JButton(new ImageIcon("./image/vaga/comment_add.png"));
	private JButton retweetButton  = new JButton(new ImageIcon("./image/vaga/comment_edit.png"));
	private JButton refreshButton  = new JButton(new ImageIcon("./image/vaga/magnify.png"));
	private JButton followButton   = new JButton(new ImageIcon("./image/vaga/user_add.png"));
	private JButton unfollowButton = new JButton(new ImageIcon("./image/vaga/user_delete.png")); 
	private JButton searchButton   = new JButton(new ImageIcon("./image/vaga/info.png"));
	
	private JButton analysisButton = new JButton(new ImageIcon("./image/vaga/image.png"));
	
	private ImageIcon image = new ImageIcon("image/GRAFFITI.jpg");
	
	private TweetPanel    tweetPanel;
	private TimelinePanel timelinePanel;
	private SearchPanel searchPanel;
	
	public FunctionPanel(Cuckoo cuc, TweetPanel tp, TimelinePanel tp2, SearchPanel pp) {
		cuckoo = cuc;
		tweetPanel = tp;
		timelinePanel = tp2;
		searchPanel = pp;
		
		tweetButton.setToolTipText("tweet");
		retweetButton.setToolTipText("retweet");
		refreshButton.setToolTipText("refresh");
		followButton.setToolTipText("follow");
		unfollowButton.setToolTipText("unfollow");
		searchButton.setToolTipText("search");
		analysisButton.setToolTipText("analysis");
		
		setLayout(new FlowLayout(FlowLayout.CENTER, 3, 3));
		add(tweetButton);
		add(retweetButton);
		add(refreshButton);
		add(followButton);
		add(unfollowButton);
		add(searchButton);
		add(analysisButton);

		tweetButton.addActionListener(new TweetButtonListener());
		refreshButton.addActionListener(new RefreshButtonListener());
		searchButton.addActionListener(new SearchButtonListener());
		followButton.addActionListener(new FollowButtonListener());
		unfollowButton.addActionListener(new UnfollowButtonListener());
		analysisButton.addActionListener(new AnalysisButtonListener());
	}
	
	private class TweetButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			//System.out.println("hello");
			
			String status = tweetPanel.getCurrentText().trim();
			if(status == null) {
				javax.swing.JOptionPane.showMessageDialog(null, "Status cannot be null!");
			} else {
				cuckoo.writeStatus(status);
			}
		}
	}
	
	private class RefreshButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			//cuckoo.updateFolloweeStatus();
			String followeeString = tweetPanel.getCurrentText().trim();
			
			if(followeeString == null) {
				javax.swing.JOptionPane.showMessageDialog(null, "Followee Id cannot be null!");
			} else {
				cuckoo.updateFolloweeStatus(followeeString);
			}
		}
	}
	
	private class SearchButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			String searchContent = tweetPanel.getCurrentText().trim();
			
			if(searchContent == null) {
				javax.swing.JOptionPane.showMessageDialog(null, "Search content cannot be null!");
			} else {
				searchPanel.search(searchContent);
			}
		}
	}
	
	private class FollowButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			String followeeString = tweetPanel.getCurrentText().trim();
			
			if(followeeString == null) {
				javax.swing.JOptionPane.showMessageDialog(null, "Followee Id cannot be null!");
			} else {
				cuckoo.follow(followeeString);
			}
		}
	}
	
	private class UnfollowButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			String followeeString = tweetPanel.getCurrentText().trim();
			
			if(followeeString == null) {
				javax.swing.JOptionPane.showMessageDialog(null, "Content cannot be null!");
			} else {
				cuckoo.unfollow(followeeString);
			}
		}
	}
	
	private class AnalysisButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			new Thread() {
	            public void run() {
	            	//trigger the traffic analysis frame
	            	TrafficAnalysisFrame trafficFrame = new TrafficAnalysisFrame(cuckoo.getTrafficAnalysisXMLDB());
	            	trafficFrame.pack();
	            	RefineryUtilities.centerFrameOnScreen(trafficFrame);
	            	trafficFrame.setVisible(true);
	            	trafficFrame.setAlwaysOnTop(true);
	            	trafficFrame.initFigure();
	            	trafficFrame.monitor();
	            	//trafficFrame.randomTest();
	            }
			}.start();
		}
	}
	
	protected void paintComponent(Graphics g) {
		setOpaque(true);
		super.paintComponent(g);
		Dimension d = getSize();
		
		for (int x = 0; x < d.width; x += image.getIconWidth())
			for (int y = 0; y < d.height; y += image.getIconHeight())
				g.drawImage(image.getImage(), x, y, null, null);
	}

}
