package guinterface;

import infrastructure.DeterministicNodeIdFactory;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import application.Profile;

import database.AccountXMLDB;
import database.ProfileXMLDB;

import rice.environment.Environment;
import rice.pastry.PastryNode;
import rice.pastry.PastryNodeFactory;
import rice.pastry.socket.SocketPastryNodeFactory;
import toolbox.Account;

import winterwell.jtwitter.OAuthSignpostClient;
import winterwell.jtwitter.Twitter;


public class LoginFrame extends JFrame {
	private static final long serialVersionUID = 201006281553L;
	
	private static final String dir = "./account";
	private static final String path = "./account/account.xml";
	
	private static final String CONFIGURE = "./configure";
	//private static final String PREFILLED = "./login_cfg";
	
	private JTextField usernameTextField;
	//private JPasswordField passwordTextField;
	private JButton loginButton    = new JButton("Login'in");
	private JButton registerButton = new JButton("Register");
	
	private ImageIcon icon = new ImageIcon("image/cuckoo.jpg");
	
	private AccountXMLDB accountDB = new AccountXMLDB(dir, path, "account");
	
	private DeterministicNodeIdFactory nidFactory = null;
	private PastryNodeFactory factory;
	private Environment env;
	private int bindport, bootport;
	private InetSocketAddress bootaddress;
	
	public LoginFrame() {
		
		//////////set cuckoo parameter/////////
		// Loads pastry settings
		env = new Environment();
		env.getParameters().setString("nat_search_policy", "never");
		
		try {
			BufferedReader is = new BufferedReader(new FileReader(CONFIGURE));
			String cfgline;
			while((cfgline = is.readLine()) != null) {
				cfgline = cfgline.trim();
				//System.out.println(cfgline.substring(0, 1));
				if(cfgline.substring(0, 2).compareTo("//") != 0) {
					String[] split = cfgline.split("#");
					bindport = Integer.valueOf(split[0]).intValue();
					String bootip = split[1];
					InetAddress bootaddr;
					if(bootip.compareTo("localhost") == 0) {
						bootaddr = InetAddress.getLocalHost();
					} else {
						bootaddr = InetAddress.getByName(bootip);
					}
					bootport = Integer.valueOf(split[2]).intValue();
					bootaddress = new InetSocketAddress(bootaddr, bootport);
				}
			}
			is.close();
			factory = new SocketPastryNodeFactory(nidFactory, bindport, env);
			
			//is = new BufferedReader(new FileReader(PREFILLED));
			//String username = is.readLine().trim();
			//String password = is.readLine().trim();
			usernameTextField = new JTextField(15);
			//passwordTextField = new JPasswordField(password, 15);
			is.close();
			
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		///////////////////////////////////////
		//setSize(350, 200);
		//setLayout(new GridBagLayout());
		
	    //GridBagConstraints gbConstraints = new GridBagConstraints();
	    //gbConstraints.fill = GridBagConstraints.CENTER;
	    //gbConstraints.anchor = GridBagConstraints.CENTER;
		
	    //Container container = getContentPane();
	    //addComp(usernameLabel, container,     gbConstraints, 0, 0, 2, 4, 1, 1);
	    //addComp(usernameTextField, container, gbConstraints, 0, 4, 2, 6, 1, 1);
	    //addComp(passwordLabel, container,     gbConstraints, 2, 0, 2, 4, 1, 1);
	    //addComp(passwordTextField, container, gbConstraints, 2, 4, 2, 6, 1, 1);
	    //addComp(loginButton, container,       gbConstraints, 4, 1, 2, 3, 1, 1);
	    //addComp(registerButton, container,    gbConstraints, 4, 6, 2, 3, 1, 1);
	    //CuckooPicPanel cuckooPicPanel = new CuckooPicPanel();
		
	    AccountPanel accountPanel = new AccountPanel(usernameTextField);
	    FuncPanel funcPanel = new FuncPanel(loginButton, registerButton);
		
	    JPanel dumpPanel = new JPanel();
	    dumpPanel.setLayout(new BorderLayout(10, 5));
	    dumpPanel.add(new CuckooPicPanel(), BorderLayout.WEST);
	    dumpPanel.add(accountPanel, BorderLayout.CENTER);
	    dumpPanel.add(new JPanel(), BorderLayout.EAST);
	    
	    add(dumpPanel, BorderLayout.CENTER);
	    add(funcPanel, BorderLayout.SOUTH);
	    
	    pack();
	    setTitle("Cuckoo");
		setLocationRelativeTo(null); // Center the frame
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setIconImage(icon.getImage());
		
	    loginButton.addActionListener(new LoginListener(this));
	    registerButton.addActionListener(new RegisterListener());
	}
	
	/*
	private void addComp(Component c, Container container,
            GridBagConstraints gbConstraints, 
            int row, int column,
            int numberOfRows, int numberOfColumns,
            double weightx, double weighty) {

		gbConstraints.gridx = column;
		gbConstraints.gridy = row;
		gbConstraints.gridwidth = numberOfColumns;
		gbConstraints.gridheight = numberOfRows;
		gbConstraints.weightx = weightx;
		gbConstraints.weighty = weighty;

		container.add(c, gbConstraints); 
	}*/
	
	private class LoginListener implements ActionListener {
		private LoginFrame loginFrame;
		
		public LoginListener(LoginFrame lf) {
			loginFrame = lf;
		}
		
		public void actionPerformed(ActionEvent arg0) {
			String um = usernameTextField.getText().trim();
			//String pw = passwordTextField.getText().trim();
			
			int r = accountDB.checkAccount(new Account(um));
			if(r == 3) {
				String userid = accountDB.getUserId(um);
				if(userid == null) {System.out.println("username is not found!!");}
				
				nidFactory = new DeterministicNodeIdFactory(userid);
				try {
					PastryNode node = factory.newNode(nidFactory.generateNodeId());
					node.boot(bootaddress);
								
					//the node may require sending several messages to fully boot into the ring
					synchronized (node) {
						while (!node.isReady() && !node.joinFailed()) {
							// delay so we don't busy-wait
							node.wait(500);
							// abort if can't join
							if (node.joinFailed()) {
								System.out.println("Could not join the FreePastry ring.  Reason:" + node.joinFailedReason());
							}
						}
					}
					System.out.println("Finished creating new node " + node + um);
				
					//CuckooAppGUI cuckoo = new CuckooAppGUI(node, userid);
					TimelineFrame tlframe = new TimelineFrame(factory, node, userid, um);
					//tlframe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
					tlframe.setVisible(true);
					loginFrame.setVisible(false);
					
				} catch(IOException ioe) {
					ioe.printStackTrace();
				} catch (InterruptedException ie) {
					ie.printStackTrace();
				}
			} else if(r == 2) {
				javax.swing.JOptionPane.showMessageDialog(null, "Incorrect password!"); 
			} else if(r == 1) {
				javax.swing.JOptionPane.showMessageDialog(null, "User not found!"); 
			}
		}
	}
	
	//Check on Twitter
	private class RegisterListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			String um = usernameTextField.getText().trim();
			//String pw = passwordTextField.getText().trim();
			
			// First, OAuth to login: Make an oauth client
	        OAuthSignpostClient oauthClient = new OAuthSignpostClient(OAuthSignpostClient.JTWITTER_OAUTH_KEY, OAuthSignpostClient.JTWITTER_OAUTH_SECRET, "oob");
	        // open the authorisation page in the user's browser
	        oauthClient.authorizeDesktop(); // Note: this only works on desktop PCs
	        // or direct the user to the webpage given jby oauthClient.authorizeUrl() 
	        // get the pin from the user since we're using "oob" instead of a callback servlet
	        String v = OAuthSignpostClient.askUser("Please enter the verification PIN from Twitter");
	        oauthClient.setAuthorizationCode(v);

	        // Make a Twitter object
	        Twitter tw = new Twitter(um, oauthClient); 
			
			Twitter.User u = null;
			//long uid = -1;
			//String headurl = null;
			
			//Twitter tw = new Twitter(um, pw);
			if(tw.isValidLogin() != true) {
				javax.swing.JOptionPane.showMessageDialog(null, "Twitter check not pass!"); 
				return;
			} else {
				u = tw.getUser(um);
				//uid = u.getId();
				//headurl = u.getProfileImageUrl().toString();
			}
			
			if(u==null) {
				System.out.println("Wrong!!!");
			}
			
			boolean s = accountDB.addAccount(new Account(um, u.getId()));
			if(s == true) {
				String  dir = "xml/" + u.getId();
				savePicToFile(u.getProfileImageUrl().toString(), "image/"+u.getId()+".jpg");
				System.out.println(u.getProfileImageUrl().toString());
				ProfileXMLDB myProfileDB = new ProfileXMLDB(dir, dir + "/myprofile.xml", "myprofile");
				myProfileDB.addProfile(new Profile(u));
				
				javax.swing.JOptionPane.showMessageDialog(null, "register successfully!"); 
			} else {
				javax.swing.JOptionPane.showMessageDialog(null, "register failed! (user already exist)");
				return;
			}
			
			System.out.println("Registration finished!");
		}
	}
	
	private class CuckooPicPanel extends JPanel {
		private static final long serialVersionUID = 201008121647L;
		
		private ImageIcon headIcon = new ImageIcon("image/cuckoo.jpg");
		private JLabel headLable = new JLabel(headIcon);
		
		public CuckooPicPanel() {
			setLayout(new BorderLayout (5, 10));
			add(headLable, BorderLayout.NORTH);
		}
	}
	
	private class AccountPanel extends JPanel {
		private static final long serialVersionUID = 201008121648L;
		
		private JLabel usernameLabel = new JLabel("Twitter User: ");
		//private JLabel passwordLabel = new JLabel("Password: ");
		
		private JLabel dummyLabel1 = new JLabel("");
		private JLabel dummyLabel2 = new JLabel("");
		private JLabel dummyLabel3 = new JLabel("");
		
		private JTextField usernameTextField;
		//private JTextField passwordTextField;
		
		public AccountPanel(JTextField utf) {
			usernameTextField = utf;
			//passwordTextField = ptf;
			
			setLayout(new GridLayout(7, 1));
			add(dummyLabel1);
			add(usernameLabel);
			add(usernameTextField);
			add(dummyLabel2);
			//add(passwordLabel);
			//add(passwordTextField);
			add(dummyLabel3);
		}
	}
	
	private class FuncPanel extends JPanel {
		private static final long serialVersionUID = 201008121651L;
		
		private JButton loginButton;
		private JButton regstButton;
		
		public FuncPanel(JButton ljb, JButton rjb) {
			loginButton = ljb;
			regstButton = rjb;
			
			setLayout(new GridLayout(1, 2));
			add(loginButton);
			add(regstButton);
		}
	}
	
	public static void savePicToFile(String destUrl, String localPath) {
		FileOutputStream fos = null;
		BufferedInputStream bis = null;
		HttpURLConnection httpUrl = null;
		URL url = null;
		int BUFFER_SIZE = 1024;

		byte[] buf = new byte[BUFFER_SIZE];
		int size = 0;
		try {
			url = new URL(destUrl);
			httpUrl = (HttpURLConnection) url.openConnection();
			httpUrl.connect();
			bis = new BufferedInputStream(httpUrl.getInputStream());

			fos = new FileOutputStream(localPath);
			while ((size = bis.read(buf)) != -1) {
				fos.write(buf, 0, size);
			}
			fos.flush();
		} catch (IOException e) {
		} catch (ClassCastException e) {
		} finally {
			try {
				fos.close();
				bis.close();
				httpUrl.disconnect();
			} catch (IOException e) {
			} catch (NullPointerException e) {
			}
		}
	}
	
	public static void main(String[] args) {
		LoginFrame loginframe = new LoginFrame();
		loginframe.setVisible(true);
		
		//savePicToFile("http://s.twimg.com/a/1281572621/images/default_profile_1_normal.png", "xml/111.jpg");
	}

}
