package message;

import java.io.IOException;

import rice.p2p.commonapi.Endpoint;
import rice.p2p.commonapi.Id;
import rice.p2p.commonapi.Message;
import rice.p2p.commonapi.NodeHandle;
import rice.p2p.commonapi.rawserialization.InputBuffer;
import rice.p2p.commonapi.rawserialization.OutputBuffer;
import rice.p2p.commonapi.rawserialization.RawMessage;

public abstract class CuckooMsg implements RawMessage {
	private static final long serialVersionUID = 201008081216L;
	
	protected Id from;
	protected Id to;
	protected String fromUserId; //from user id
	protected String toUserId;   //to user id
    
	protected NodeHandle nh;
	
	// Constructor
	public CuckooMsg(Id from, Id to, String fid, String tid, NodeHandle nh) {
    	this.from = from;
    	this.to = to;
    	
    	this.fromUserId = fid;
    	this.toUserId = tid;
    	
    	this.nh = nh;
    }
	
	//@Override: required by the "RawMessage" Interface 
	public void serialize(OutputBuffer buf) throws IOException {
		buf.writeShort(from.getType());
	    from.serialize(buf);
	    
	    buf.writeShort(to.getType());
	    to.serialize(buf);
	    
	    buf.writeUTF(fromUserId);
	    buf.writeUTF(toUserId);
	    
	    nh.serialize(buf);
	}
	
	/**
	 * Deserializing constructor.  This does the "real" deserialization.
	 * 
	 * @param buf
	 * @param endpoint
	 * @throws IOException
	 */
	public CuckooMsg(InputBuffer buf, Endpoint endpoint) throws IOException {
		from = endpoint.readId(buf, buf.readShort()); 
	    to = endpoint.readId(buf, buf.readShort());
	    fromUserId = buf.readUTF();
	    toUserId = buf.readUTF();
	    nh = endpoint.readNodeHandle(buf);
	}
	
	/***************getters************************/
	public int getPriority() {
		return Message.LOW_PRIORITY;
	}
    
    public NodeHandle getNodeHandle() {
		return nh;
	}
	
	public Id getTo() {
		return to;
	}
	
	public Id getFrom() {
		return from;
	}
	
	public String getToUserId() {
		return toUserId;
	}
	
	public String getFromUserId() {
		return fromUserId;
	}
}
